SessionError

Enum SessionError 

pub enum SessionError {
    IntoSession,
    Serialize(Error),
    Deserialize(Error),
    PayloadTooLong(usize),
    CryptoError,
    Storage,
    ReachedMaxChannels,
    IdentifierMismatch,
    OutputBufferShort,
    Import,
}
Session related errors

Variants



IntoSession

An error during session creation



Serialize(Error)

An error during serialization using TLSCodec



Deserialize(Error)

An error during deserialization using TLSCodec



PayloadTooLong(usize)

The given channel payload exceeded the maximum length



CryptoError

An error in an underlying cryptographic primitive



Storage

An error in storing or loading a session state



ReachedMaxChannels

The maxmium number of derivable channels has been reached



IdentifierMismatch

A channel message contains an inappropriate channel identifier



OutputBufferShort

The given payload exceeds the available output buffer



Import

An error arising during the import of an external secret